﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Xml;
using System.IO;
using Microsoft.SharePoint.Utilities;

namespace TheBlackKnightSings
{
    class CleanupWebPartsFeatureReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            using (SPSite site = properties.Feature.Parent as SPSite)
            {
                RemoveWebParts(site, properties.Definition);
            }
        }

        private void RemoveWebParts(SPSite site, SPFeatureDefinition definition)
        {
            using (SPWeb web = site.RootWeb)
            {
                string wpcatalogUrl = SPUrlUtility.CombineUrl(web.Url, "_catalogs/wp");

                // Get the Feature.xml for the feature
                //
                XmlDocument featureXml = new XmlDocument();
                featureXml.LoadXml(definition.GetXmlDefinition(web.Locale).OuterXml);

                XmlNamespaceManager nsMgr = new XmlNamespaceManager(featureXml.NameTable);
                nsMgr.AddNamespace("sp", "http://schemas.microsoft.com/sharepoint/");

                // Get the Location attribute of each ElementManifest inside each ElementManifests inside the Feature
                //
                foreach (XmlNode locationNode in featureXml.SelectNodes("/sp:Feature/sp:ElementManifests/sp:ElementManifest/@Location", nsMgr))
                {
                    // Get the ElementManifest XML
                    //
                    XmlDocument elementManifest = new XmlDocument();
                    elementManifest.Load(Path.Combine(definition.RootDirectory, locationNode.Value));

                    XmlNamespaceManager nsMgr2 = new XmlNamespaceManager(elementManifest.NameTable);
                    nsMgr2.AddNamespace("sp", "http://schemas.microsoft.com/sharepoint/");

                    // Get the Url attribute of each file in Modules (with List attribute='113' WebPartCatalog
                    //
                    foreach (XmlNode webPartFileUrl in elementManifest.SelectNodes("/sp:Elements/sp:Module[@List='113']/sp:File/@Url", nsMgr2))
                    {
                        // Delete the file from the WebPart Catalog
                        //
                        SPFile wpFile = web.GetFile(SPUrlUtility.CombineUrl(wpcatalogUrl,webPartFileUrl.Value));
                        wpFile.Delete();
                    }

                }
            }
        }

        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
        }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
            // HACK:
            // As solution retraction just uninstalls features without first deactivating
            // them, the code associated with FeatureDeactivating isn't executed unless
            // we deactivate the features ourselves
            // This code runs through all site and deactivates the feature on uninstall
            //
            foreach (SPWebService webService in new SPWebServiceCollection(SPFarm.Local))
            {
                foreach (SPWebApplication webApp in webService.WebApplications)
                {
                    foreach (SPSite site in webApp.Sites)
                    {
                        try
                        {
                            foreach (SPFeature feature in site.Features)
                            {
                                if (feature.DefinitionId == properties.Definition.Id)
                                {
                                    site.Features.Remove(properties.Definition.Id);
                                    break;
                                }
                            }
                        }
                        finally
                        {
                            site.Dispose();
                        }
                    }
                }
            }
        }
    }
}
